type fst
type state_id

external test_fst_lib   : string -> unit = "testFstLib"

external new_fst        : unit -> fst * state_id = "newFst"
external add_state      : fst -> state_id = "addState"
external set_final      : fst -> state_id -> unit = "setFinal"
external add_arc        : fst -> state_id -> state_id -> char -> char -> unit = "addArc"

external compose        : fst -> fst -> fst = "compose"

(* these next four modify their first argument *)
external union          : fst -> fst -> unit = "union_"
external concat         : fst -> fst -> unit = "concat"
external project_input  : fst -> unit = "projectInput"
external project_output : fst -> unit = "projectOutput"

external shortest_path  : fst -> fst = "shortestPath"
external write_to_file  : fst -> string -> bool = "writeToFile"
external read_from_file : string -> fst = "readFromFile"

let rec replicate m x = match m with
    | 0 -> []
    | _ -> x :: replicate (m-1) x
let rec enumerate' m l = match l with
    | []      -> []
    | (x::xs) -> (m, x) :: enumerate' (m+1) xs
let enumerate = enumerate' 0

let const_fst win wout =
    let (fst, start) = new_fst () in
    let statesIn  = List.map add_state (replicate (String.length win ) fst) in
    let statesOut = List.map add_state (replicate (String.length wout) fst) in
    List.iter2 (fun (n, a) b -> add_arc fst a b win.[n]  (char_of_int 0))
               (enumerate (start::statesIn)) statesIn;
    List.iter2 (fun (n, a) b -> add_arc fst a b (char_of_int 0) wout.[n])
               (enumerate (start::statesOut)) statesOut;
    fst
