#include "OpenFst/fst/lib/fstlib.h"
#include "OpenFst/fst/lib/vector-fst.h"

typedef fst::StdVectorFst Fst;
typedef fst::StdArc       Arc;

extern "C" {
    void TestFstLib(char *filename) {
        Fst fst;
        fst.AddState();
        fst.SetStart(0);
        fst.AddArc(0, Arc(1,1,1.0,1));
        fst.AddArc(0, Arc(2,2,1.0,1));
        fst.AddState();
        fst.AddArc(1, Arc(3,3,1.0,2));
        fst.AddState();
        fst.SetFinal(2, 1.0);
        fst.Write(filename);
    }

    Fst *NewFst() {
      Fst *out = new Fst();
      out->AddState();
      out->SetStart(0);
      return out;
    }

    void FinalizeFst(Fst *in) {
      delete in;
    }

    int AddState(Fst *in) {
      return in->AddState();
    }

    void SetFinal(Fst *in, int state) {
      in->SetFinal(state, 1);
    }

    void AddArc(Fst *in, int stateFrom, int stateTo, int charFrom, int charTo) {
      in->AddArc(stateFrom, Arc(charFrom, charTo, 1, stateTo));
    }

    Fst *Compose(Fst *inl, Fst *inr) {
      Fst *out = new Fst();
      fst::ArcSort(inr, fst::ILabelCompare<Arc>());
      fst::Compose(*inl, *inr, out);
      return out;
    }

    void Union(Fst *inl, Fst *inr) {
      fst::Union(inl, *inr);
    }

    void Concat(Fst *inl, Fst *inr) {
      fst::Concat(inl, *inr);
    }

    void ProjectInput(Fst *in) {
      fst::Project(in, fst::PROJECT_INPUT);
    }

    void ProjectOutput(Fst *in) {
      fst::Project(in, fst::PROJECT_OUTPUT);
    }

    Fst *ShortestPath(Fst *in) {
      Fst *path = new Fst();
      fst::ShortestPath(*in, path);
      return path;
    }

    int  WriteToFile(Fst *in, char *filename) {
      return in->Write(filename);
    }

    Fst *ReadFromFile(char *filename) {
      return Fst::Read(filename);
    }
}
