#include "wrapperOcamlFromC.h"
#include "wrapperCFromCpp.h"

static struct custom_operations fst_operations = {
    "www.dmwit.com.open_fst",
    finalizeFst,
    custom_compare_default,
    custom_hash_default,
    custom_serialize_default,
    custom_deserialize_default
};

CAMLprim value testFstLib(value filename) {
    CAMLparam1(filename);
    TestFstLib(String_val(filename));
    CAMLreturn(Val_unit);
}

CAMLprim value newFst(value unit) {
    CAMLparam1(unit);
    CAMLlocal2(fstBlock, pairBlock);
    fstBlock = caml_alloc_custom(&fst_operations, sizeof(struct Fst **), 1, 1000);
    fstValue(fstBlock) = NewFst();
    pairBlock = caml_alloc_tuple(2);
    Store_field(pairBlock, 0, fstBlock);
    Store_field(pairBlock, 1, Val_int(0));
    CAMLreturn(pairBlock);
}

void finalizeFst(value fst) {
    FinalizeFst(fstValue(fst));
}

CAMLprim value addState(value fst) {
    CAMLparam1(fst);
    CAMLlocal1(state);
    state = Val_int(AddState(fstValue(fst)));
    CAMLreturn(state);
}

CAMLprim value setFinal(value fst, value state) {
    CAMLparam2(fst, state);
    SetFinal(fstValue(fst), Int_val(state));
    CAMLreturn(Val_unit);
}

CAMLprim value addArc(value fst, value stateFrom, value stateTo, value charFrom, value charTo) {
    CAMLparam5(fst, stateFrom, stateTo, charFrom, charTo);
    AddArc(fstValue(fst), Int_val(stateFrom), Int_val(stateTo), Int_val(charFrom), Int_val(charTo));
    CAMLreturn(Val_unit);
}

CAMLprim value compose(value fstl, value fstr) {
    CAMLparam2(fstl, fstr);
    CAMLlocal1(fstBlock);
    fstBlock = caml_alloc_custom(&fst_operations, sizeof(struct Fst **), 1, 1000);
    fstValue(fstBlock) = Compose(fstValue(fstl), fstValue(fstr));
    CAMLreturn(fstBlock);
}

CAMLprim value union_(value fstl, value fstr) {
    CAMLparam2(fstl, fstr);
    Union(fstValue(fstl), fstValue(fstr));
    CAMLreturn(Val_unit);
}

CAMLprim value concat(value fstl, value fstr) {
    CAMLparam2(fstl, fstr);
    Concat(fstValue(fstl), fstValue(fstr));
    CAMLreturn(Val_unit);
}

CAMLprim value projectInput(value fst) {
    CAMLparam1(fst);
    ProjectInput(fstValue(fst));
    CAMLreturn(Val_unit);
}

CAMLprim value projectOutput(value fst) {
    CAMLparam1(fst);
    ProjectOutput(fstValue(fst));
    CAMLreturn(Val_unit);
}

CAMLprim value shortestPath(value fstIn) {
    CAMLparam1(fstIn);
    CAMLlocal1(fstOut);
    fstOut = caml_alloc_custom(&fst_operations, sizeof(struct Fst **), 1, 1000);
    fstValue(fstOut) = ShortestPath(fstValue(fstIn));
    CAMLreturn(fstOut);
}

/* for now, we don't worry about filenames with '\0' in them; let
 * things fail catastrophically for people who try to do something
 * as stupid as that */
CAMLprim value writeToFile(value fst, value filename) {
    CAMLparam2(fst, filename);
    CAMLreturn(Val_bool(WriteToFile(fstValue(fst), String_val(filename))));
}

CAMLprim value readFromFile(value filename) {
    CAMLparam1(filename);
    CAMLlocal1(fstBlock);
    fstBlock = caml_alloc_custom(&fst_operations, sizeof(struct Fst **), 1, 1000);
    fstValue(fstBlock) = ReadFromFile(String_val(filename));
    CAMLreturn(fstBlock);
}
